//  SearchLoad Options: A Firefox addon that extends the capabilities of the searchbar.
//  Copyright (C) 2012  Esteban Torres
//  
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

searchLoad_Preferences = {
    pref: {},

    prefObserver: {

        branch   : null,

        register : function(){
            let prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                        .getService(Components.interfaces.nsIPrefService);

            this.branch = prefService.getBranch("extensions.searchload.");

            this.branch.addObserver("", this, false);
        },

        unregister : function() {
            this.branch.removeObserver("", this);
        },

        observe : function(aSubject, aTopic, aData) {
            switch(aData){
                case "clearonsearch" : searchLoad_Preferences.pref.clearonsearch  = this.branch.getBoolPref("clearonsearch" ); break;
                case "cleardelay"    : searchLoad_Preferences.pref.cleardelay     = this.branch.getIntPref ("cleardelay"    );
                                       searchLoad_Preferences.pref.cleardelay *= 1000;
                                       break;
                case "resetonsearch" : searchLoad_Preferences.pref.resetonsearch  = this.branch.getBoolPref("resetonsearch" ); break;
                case "resetdelay"    : searchLoad_Preferences.pref.resetdelay     = this.branch.getIntPref ("resetdelay"    );
                                       searchLoad_Preferences.pref.resetdelay *= 1000;
                                       break;
                case "cancelclear"   : searchLoad_Preferences.pref.cancelclear    = this.branch.getBoolPref("cancelclear"   ); break;
                case "cancelreset"   : searchLoad_Preferences.pref.cancelreset    = this.branch.getBoolPref("cancelreset"   ); break;
                case "defaultengine" : searchLoad_Preferences.pref.defaultengine  = this.branch.getCharPref("defaultengine" );
                                       searchLoad_Preferences.pref.defaultengine  = decodeURIComponent( escape(searchLoad_Preferences.pref.defaultengine) );
                                       break;
                case "newtab"        : searchLoad_Preferences.pref.newtab         = this.branch.getIntPref ("newtab"        ); break;
                case "detectblanktab": searchLoad_Preferences.pref.detectblanktab = this.branch.getBoolPref("detectblanktab"); break;
                case "detecthometab" : searchLoad_Preferences.pref.detecthometab  = this.branch.getBoolPref("detecthometab" ); break;
                case "detectnewtab"  : searchLoad_Preferences.pref.detectnewtab   = this.branch.getBoolPref("detectnewtab"  ); break;
            }
        }
    }
};

function searchLoad_Options(win)
{
    var self      = this;
    var window    = win;
    var searchbar = win.document.getElementById("searchbar");

    this.MOZhandleSearch = null;
    this.MOZdoSearch = null;
    this.MOZToolboxCustomizeDone = null;
    this.eventModifier = false;
    this.eventShift = false;
    this.clearTID = null;//Timeout ID
    this.resetTID = null;//Timeout ID

    this.MOZToolboxCustomizeDone = window.BrowserToolboxCustomizeDone;
    window.BrowserToolboxCustomizeDone = function(aToolboxChanged){
        self.MOZToolboxCustomizeDone.apply(this,arguments);
        self.init();
    };

    let toolbox = window.document.getElementById("navigator-toolbox");
    toolbox.customizeDone = window.BrowserToolboxCustomizeDone;

    this.configure = function(){
        var winManager = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                                   .getService(Components.interfaces.nsIWindowMediator);
        var recentWin = winManager.getMostRecentWindow("SearchLoadOptions:Preferences");
        if( recentWin )
            recentWin.focus();
        else
            window.openDialog("chrome://searchload/content/preferences.xul", "", "dialog,centerscreen");
    };

    this.init = function(){
        if(!searchbar)
            return;

        let stringBundle = window.document.getElementById("searchloadoptions-stringbundle");
        if( !stringBundle )
        {
            let bundleset = window.document.getElementById("stringbundleset");
            stringBundle = window.document.createElement("stringbundle");
            stringBundle.setAttribute("id","searchloadoptions-stringbundle");
            stringBundle.setAttribute("src","chrome://searchload/locale/searchbar.properties");
            bundleset.appendChild( stringBundle );
        }

        if( searchbar._popup && !window.document.getElementById("searchloadoptions-menuitem") )
        {
            let element = window.document.createElement("menuitem");
            element.setAttribute("class","open-engine-manager");
            element.setAttribute("id","searchloadoptions-menuitem");
            element.setAttribute("label",stringBundle.getString("searchoptions.label"));
            element.addEventListener('command', this.configure, true );
            searchbar._popup.appendChild(element);
        }

        if( !this.MOZhandleSearch )
        {
            this.MOZhandleSearch = searchbar.handleSearchCommand;

            searchbar.handleSearchCommand = function(aEvent){
                self.eventModifier = aEvent&&(aEvent.altKey||aEvent.ctrlKey||aEvent.shiftKey||aEvent.button==1);
                self.eventShift    = aEvent&&aEvent.shiftKey;
                self.MOZhandleSearch.apply(this,arguments);
                self.eventModifier = false;
                self.eventShift    = false;
            };
        }

        if( !this.MOZdoSearch )
        {
            this.MOZdoSearch = searchbar.doSearch;

            searchbar.doSearch = function(aData,aWhere){
                if( self.eventModifier )
                    switch( searchLoad_Preferences.pref.newtab )
                    {
                        case 0:  aWhere=(self.eventShift)?"tabshifted":"tab"; break;
                        case 1:  aWhere=(self.eventShift)?"tabshifted":"current"; break;
                        case 2:  aWhere=(self.eventShift)?"tab":"current"; break;
                        default: aWhere="tab";
                    }
                else
                {
                    let currentURL = window.getWebNavigation().currentURI.spec;
                    let homeTabDetected = false;
                    if(searchLoad_Preferences.pref.detecthometab)
                    {
                        let homepages = window.gPrefService.getComplexValue("browser.startup.homepage",Components.interfaces.nsIPrefLocalizedString).data;
                            homepages = homepages.split("|");
                        currentURL = currentURL.replace(/(^\s+)|(\s+$)|(^https?:\/\/)|(\/$)/g,"").replace(/^www\./,"");
                        for(let i=0; i<homepages.length; i++)
                            if( currentURL==homepages[i].replace(/(^\s+)|(\s+$)|(^https?:\/\/)|(\/$)/g,"").replace(/^www\./,"") )
                            {
                                homeTabDetected = true;
                                break;
                            }
                    }
                    if( searchLoad_Preferences.pref.newtab==0 ||
                        ( searchLoad_Preferences.pref.detectnewtab && currentURL==window.gPrefService.getCharPref("browser.newtab.url") ||
                          searchLoad_Preferences.pref.detectblanktab && currentURL=="about:blank" ||
                          searchLoad_Preferences.pref.detecthometab && homeTabDetected 
                        ) &&
                       !window.getWebNavigation().nsIWebProgress.isLoadingDocument
                      )
                        aWhere="current";
                    else switch( searchLoad_Preferences.pref.newtab )
                    {
                        case 1: aWhere="tab"; break;
                        case 2: aWhere="tabshifted"; break;
                        default: aWhere="tab";
                    }
                }

                //clear searchbar
                //
                if(searchLoad_Preferences.pref.clearonsearch)
                {
                    if(searchLoad_Preferences.pref.cleardelay==0)
                        this.value = '';
                    else if(searchLoad_Preferences.pref.cleardelay>0)
                    {
                        window.clearTimeout(self.clearTID);
                        self.clearTID = window.setTimeout(
                            function(){
                                if( !searchLoad_Preferences.pref.cancelclear || (!searchbar.textbox.hasAttribute("focused") && !(searchbar._popup.state=="open")) )
                                {
                                    searchbar.value = '';

                                    let changeEvent = window.document.createEvent('Event');
                                    changeEvent.initEvent('input', true, true);
                                    window.setTimeout(  function(){searchbar.dispatchEvent(changeEvent);}, 0 );
                                }
                                self.clearTID = null;
                            },searchLoad_Preferences.pref.cleardelay);
                    }
                }//clear

                self.MOZdoSearch.apply(this, arguments);

                //reset searchbar
                //
                if(searchLoad_Preferences.pref.resetonsearch)
                {
                    let selectedEngine = this.currentEngine.name;

                    if(selectedEngine != searchLoad_Preferences.pref.defaultengine)
                    {
                            if(searchLoad_Preferences.pref.resetdelay==0)
                            {
                                if( this.searchService.getEngineByName(searchLoad_Preferences.pref.defaultengine) )
                                    this.currentEngine = this.searchService.getEngineByName(searchLoad_Preferences.pref.defaultengine);
                            }
                            else if(searchLoad_Preferences.pref.resetdelay>0)
                            {
                                window.clearTimeout(self.resetTID);
                                self.resetTID = window.setTimeout(
                                    function(){
                                        if( !searchLoad_Preferences.pref.cancelreset || (!searchbar.textbox.hasAttribute("focused") && !(searchbar._popup.state=="open")) )
                                        {
                                            if( searchbar.searchService.getEngineByName(searchLoad_Preferences.pref.defaultengine) )
                                                searchbar.currentEngine = searchbar.searchService.getEngineByName(searchLoad_Preferences.pref.defaultengine);

                                            let changeEvent = window.document.createEvent('Event');
                                            changeEvent.initEvent('input', true, true);
                                            window.setTimeout( function(){searchbar.dispatchEvent(changeEvent);}, 0 );
                                        }
                                        self.resetTID = null;
                                    },searchLoad_Preferences.pref.resetdelay);
                            }
                    }
                }//reset
            };//doSearch
        }
    };//init

    this.shutdown = function(){

        if( this.MOZToolboxCustomizeDone )
            window.BrowserToolboxCustomizeDone = this.MOZToolboxCustomizeDone;

        let toolbox = window.document.getElementById("navigator-toolbox");

        if( this.MOZToolboxCustomizeDone )
            toolbox.customizeDone = this.MOZToolboxCustomizeDone;

        if( !searchbar )
            return;

        let menuitem = window.document.getElementById("searchloadoptions-menuitem");
        if( menuitem )
        {
            menuitem.removeEventListener('command', this.configure, true );
            menuitem.parentNode.removeChild( menuitem );
        }

        searchbar.doSearch = this.MOZdoSearch;
        searchbar.handleSearchCommand = this.MOZhandleSearch;

        let stringBundle = window.document.getElementById("searchloadoptions-stringbundle");
        if( stringBundle )
            stringBundle.parentNode.removeChild(stringBundle);
    };
}

function loadIntoWindow(window)
{
    window.searchLoad_Options = new searchLoad_Options(window);
    window.searchLoad_Options.init();
}

function unloadFromWindow(window)
{
    window.searchLoad_Options.shutdown();
    window.searchLoad_Options = null;
}

var windowListener = {
    onOpenWindow: function(aWindow)
    {
        let domWindow = aWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                               .getInterface(Components.interfaces.nsIDOMWindow);
        domWindow.addEventListener("load", function(){
            domWindow.removeEventListener("load", arguments.callee, false);

            if( !domWindow.BrowserToolboxCustomizeDone )
                return;

            domWindow.setTimeout( loadIntoWindow, 0, domWindow );

        }, false);
    },
    onCloseWindow: function(aWindow)
    {
        let domWindow = aWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor).getInterface(Components.interfaces.nsIDOMWindow);

        if( !domWindow.BrowserToolboxCustomizeDone )
            return;

        unloadFromWindow(domWindow);
    },
    onWindowTitleChange: function(aWindow, aTitle) {}
};

function startup(data, reason) {
    searchLoad_Preferences.prefObserver.register();

    try{searchLoad_Preferences.prefObserver.branch.getBoolPref("clearonsearch" );}catch(e){searchLoad_Preferences.prefObserver.branch.setBoolPref("clearonsearch" , true );}
    try{searchLoad_Preferences.prefObserver.branch.getIntPref ("cleardelay"    );}catch(e){searchLoad_Preferences.prefObserver.branch.setIntPref ("cleardelay"    , 5    );}
    try{searchLoad_Preferences.prefObserver.branch.getBoolPref("resetonsearch" );}catch(e){searchLoad_Preferences.prefObserver.branch.setBoolPref("resetonsearch" , true );}
    try{searchLoad_Preferences.prefObserver.branch.getIntPref ("resetdelay"    );}catch(e){searchLoad_Preferences.prefObserver.branch.setIntPref ("resetdelay"    , 5    );}
    try{searchLoad_Preferences.prefObserver.branch.getBoolPref("cancelclear"   );}catch(e){searchLoad_Preferences.prefObserver.branch.setBoolPref("cancelclear"   , true );}
    try{searchLoad_Preferences.prefObserver.branch.getBoolPref("cancelreset"   );}catch(e){searchLoad_Preferences.prefObserver.branch.setBoolPref("cancelreset"   , true );}
    try{searchLoad_Preferences.prefObserver.branch.getCharPref("defaultengine" );}catch(e){searchLoad_Preferences.prefObserver.branch.setCharPref("defaultengine" , ""   );}
    try{searchLoad_Preferences.prefObserver.branch.getIntPref ("newtab"        );}catch(e){searchLoad_Preferences.prefObserver.branch.setIntPref ("newtab"        , 1    );}
    try{searchLoad_Preferences.prefObserver.branch.getBoolPref("detectblanktab");}catch(e){searchLoad_Preferences.prefObserver.branch.setBoolPref("detectblanktab", true );}
    try{searchLoad_Preferences.prefObserver.branch.getBoolPref("detecthometab" );}catch(e){searchLoad_Preferences.prefObserver.branch.setBoolPref("detecthometab" , false);}
    try{searchLoad_Preferences.prefObserver.branch.getBoolPref("detectnewtab"  );}catch(e){searchLoad_Preferences.prefObserver.branch.setBoolPref("detectnewtab"  , true );}

    searchLoad_Preferences.pref.clearonsearch =searchLoad_Preferences.prefObserver.branch.getBoolPref("clearonsearch" );
    searchLoad_Preferences.pref.cleardelay    =searchLoad_Preferences.prefObserver.branch.getIntPref ("cleardelay"    );
    searchLoad_Preferences.pref.resetonsearch =searchLoad_Preferences.prefObserver.branch.getBoolPref("resetonsearch" );
    searchLoad_Preferences.pref.resetdelay    =searchLoad_Preferences.prefObserver.branch.getIntPref ("resetdelay"    );
    searchLoad_Preferences.pref.cancelclear   =searchLoad_Preferences.prefObserver.branch.getBoolPref("cancelclear"   );
    searchLoad_Preferences.pref.cancelreset   =searchLoad_Preferences.prefObserver.branch.getBoolPref("cancelreset"   );
    searchLoad_Preferences.pref.defaultengine =searchLoad_Preferences.prefObserver.branch.getCharPref("defaultengine" );
    searchLoad_Preferences.pref.newtab        =searchLoad_Preferences.prefObserver.branch.getIntPref ("newtab"        );
    searchLoad_Preferences.pref.detectblanktab=searchLoad_Preferences.prefObserver.branch.getBoolPref("detectblanktab");
    searchLoad_Preferences.pref.detecthometab =searchLoad_Preferences.prefObserver.branch.getBoolPref("detecthometab" );
    searchLoad_Preferences.pref.detectnewtab  =searchLoad_Preferences.prefObserver.branch.getBoolPref("detectnewtab"  );

    searchLoad_Preferences.pref.cleardelay *= 1000;
    searchLoad_Preferences.pref.resetdelay *= 1000;
    searchLoad_Preferences.pref.defaultengine = decodeURIComponent( escape(searchLoad_Preferences.pref.defaultengine) );

    let wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    let enumerator = wm.getEnumerator("navigator:browser");

    while (enumerator.hasMoreElements())
    {
        let win = enumerator.getNext().QueryInterface(Components.interfaces.nsIDOMWindow);

        if( !win.BrowserToolboxCustomizeDone )
            return;

        win.setTimeout( loadIntoWindow, 0, win );
    }

    wm.addListener(windowListener);
}

function shutdown(data, reason) {
    if (reason == APP_SHUTDOWN)
        return;

    searchLoad_Preferences.prefObserver.unregister();

    let wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);

    wm.removeListener(windowListener);

    let enumerator = wm.getEnumerator("navigator:browser");

    while (enumerator.hasMoreElements())
    {
        let win = enumerator.getNext().QueryInterface(Components.interfaces.nsIDOMWindow);

        if( !win.BrowserToolboxCustomizeDone )
            return;

        unloadFromWindow(win);
    }

    searchLoad_Options = null;
}

function install(data, reason) {
    let prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try{prefs.getBoolPref("extensions.searchload.clearonsearch" );}catch(e){prefs.setBoolPref("extensions.searchload.clearonsearch" , true  );}
    try{prefs.getIntPref ("extensions.searchload.cleardelay"    );}catch(e){prefs.setIntPref ("extensions.searchload.cleardelay"    , 5     );}
    try{prefs.getBoolPref("extensions.searchload.resetonsearch" );}catch(e){prefs.setBoolPref("extensions.searchload.resetonsearch" , true  );}
    try{prefs.getIntPref ("extensions.searchload.resetdelay"    );}catch(e){prefs.setIntPref ("extensions.searchload.resetdelay"    , 5     );}
    try{prefs.getBoolPref("extensions.searchload.cancelclear"   );}catch(e){prefs.setBoolPref("extensions.searchload.cancelclear"   , true  );}
    try{prefs.getBoolPref("extensions.searchload.cancelreset"   );}catch(e){prefs.setBoolPref("extensions.searchload.cancelreset"   , true  );}
    try{prefs.getCharPref("extensions.searchload.defaultengine" );}catch(e){prefs.setCharPref("extensions.searchload.defaultengine" , ""    );}
    try{prefs.getIntPref ("extensions.searchload.newtab"        );}catch(e){prefs.setIntPref ("extensions.searchload.newtab"        , 1     );}
    try{prefs.getBoolPref("extensions.searchload.detectblanktab");}catch(e){prefs.setBoolPref("extensions.searchload.detectblanktab", true  );}
    try{prefs.getBoolPref("extensions.searchload.detecthometab" );}catch(e){prefs.setBoolPref("extensions.searchload.detecthometab" , false );}
    try{prefs.getBoolPref("extensions.searchload.detectnewtab"  );}catch(e){prefs.setBoolPref("extensions.searchload.detectnewtab"  , true  );}
}

function uninstall(data, reason) {
    if( reason == ADDON_UNINSTALL )
    {
        let prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

        prefs.deleteBranch("extensions.searchload.");
    }
}
